# core.py - 日志
# - 使用方法
# from monori_logger import MinoriLogger
# - get_logger()的参数：
#   - logger_name   default: "minori"
# LOGGER = MinoriLogger().get_logger()
# LOGGER.debug("平常的输出。代替print()")
# LOGGER.info("为了诊断问题而详细记录")
# LOGGER.warning("程序无法处理这种情形，但是应当记录此事件")
# LOGGER.exception("报告未抛出异常而抑制了此错误。程序抛出了异常。")
# LOGGER.error("报告未抛出异常而抑制了此错误。严重的错误。")
# LOGGER.critical("报告未抛出异常而抑制了此错误。非常严重的错误。")


# https://docs.python.org/3/howto/logging.html#basic-logging-tutorial
# - CRITICAL > ERROR > WARNING > INFO > DEBUG > NOTSET
# - print()         平常的输出
# - logging.debug()
# - logging.info()  为了诊断问题而详细记录
# - warning.warn()  可避免的错误，应当消除的错误
# - logging.warning 程序无法处理这种情形，但是应当记录此事件
# - raise Exception("??")   报告一个关于特定的运行时事件的错误
# - logging.error(), logging.exception, logging.critical()  报告未抛出异常而抑制了此错误

import os
import shutil
from datetime import date
import logging  # 日志


class MinoriLogger:
    # logger-dict
    _LOGGER_DICT = {}

    # 可以根据需要自主修改的部分
    _LOGGER_NAME = "log"    # defalt: "log"
    _FILENAME_BASE = "log"  # default: "log"
    _FILENAME_EXT = "log"   # default: ".log"
    _ENCODING = "utf-8"     # default: "utf-8"
    _LOG_DIR = "log"        # default: "log"
    # end

    @classmethod
    def get_logger(cls, logger_name=_LOGGER_NAME):
        # 返回默认的LOGGER
        if not cls._LOGGER_DICT.get(logger_name) is None:
            return cls._LOGGER_DICT[logger_name]

        filename_base = logger_name
        filename_ext = cls._FILENAME_EXT
        encoding = cls._ENCODING
        log_dir = cls._LOG_DIR

        if not os.path.exists(log_dir):  # or os.path.isdir('log'):
            # log作为文件或文件夹都不存在
            os.mkdir(log_dir)
            # _log_dir = log_dir
        elif os.path.isdir(log_dir):
            # log是文件夹
            # _log_dir = log_dir
            pass
        else:
            # log是文件
            log_dir = ""
            pass

        # 创建或使用当日的空的日志文件
        # - 如果文件存在，且不是今天创建的。
        # - **如果该文件名称是目录，则任其抛出异常**
        _INFOH_FILENAME = os.path.join(log_dir, "{base}-{today}.{ext}".format(
            base=filename_base, ext=filename_ext, today=date.today()))

        # 处理旧的日志文件
        _ERRORH_FILENAME = os.path.join(
            log_dir, "{base}-error-{today}.{ext}".format(base=filename_base, ext=filename_ext, today=date.today()))

        # 处理旧的日志文件
        _CRITICALH_FILENAME = os.path.join(
            log_dir, "{base}-critical-{today}.{ext}".format(base=filename_base, ext=filename_ext, today=date.today()))

        # 创建日志记录器
        cls._LOGGER_DICT[logger_name] = logging.getLogger(logger_name)
        cls._LOGGER_DICT[logger_name].setLevel(logging.DEBUG)

        # console handler
        console_handler = logging.StreamHandler()
        console_handler.setLevel(logging.DEBUG)
        # - 格式化器
        consoleh_formatter = logging.Formatter(
            # 格式：时间 日志级别 日志消息 文件路径(line 行号)
            "%(asctime)s %(levelname)-8s %(message)s %(pathname)s(line %(lineno)d)")
        # - 设置格式化器
        console_handler.setFormatter(consoleh_formatter)
        # - 添加handler
        cls._LOGGER_DICT[logger_name].addHandler(console_handler)

        # info-handler
        info_handler = logging.FileHandler(
            filename=_INFOH_FILENAME, encoding=encoding)
        # 日志级别
        info_handler.setLevel(logging.INFO)
        # 日志格式
        infoh_formatter = consoleh_formatter
        info_handler.setFormatter(infoh_formatter)
        cls._LOGGER_DICT[logger_name].addHandler(info_handler)

        # error-handler
        error_handler = logging.FileHandler(
            filename=_ERRORH_FILENAME, encoding=encoding)
        # 日志级别
        error_handler.setLevel(logging.ERROR)
        # 日志格式
        errorh_formatter = consoleh_formatter
        error_handler.setFormatter(errorh_formatter)
        cls._LOGGER_DICT[logger_name].addHandler(error_handler)

        # critical-handler
        critical_handler = logging.FileHandler(
            filename=_CRITICALH_FILENAME, encoding=encoding)
        # 日志格式
        critical_handler.setLevel(logging.CRITICAL)
        # 日志格式
        criticalh_formatter = consoleh_formatter
        critical_handler.setFormatter(criticalh_formatter)
        cls._LOGGER_DICT[logger_name].addHandler(critical_handler)

        return cls._LOGGER_DICT[logger_name]
